﻿# sample command lines:
# SetupPrn.exe /INSTDIR:"path_to_inst_dir" /ACTION:INSTALL
# SetupPrn.exe /INSTDIR:"path_to_inst_dir" /ACTION:UNINSTALL

Unicode true

!include "FileFunc.nsh"
!insertmacro GetParameters
!insertmacro GetOptions

!define PRINTER_NAME "Your printer name"
!define PREFIX "Your file prefix"

# Available paper forms by default
# "DMPAPER_LETTER",    Letter 8 1/2 x 11 in
# "DMPAPER_LEGAL",     Legal 8 1/2 x 14 in
# "DMPAPER_STATEMENT", Statement 5 1/2 x 8 1/2 in
# "DMPAPER_EXECUTIVE", Executive 7 1/4 x 10 1/2 in
# "A0"                 A0 841 x 1189 mm
# "A1"                 A1 594 x 841 mm
# "DMPAPER_A2",        A2 420 x 594 mm
# "DMPAPER_A3",        A3 297 x 420 mm
# "DMPAPER_A4",        A4 210 x 297 mm
# "DMPAPER_A5",        A5 148 x 210 mm
# "DMPAPER_B4",        B4 (JIS) 250 x 354
# "DMPAPER_B5",        B5 (JIS) 182 x 257 mm
# "ThermalReceipt",    Thermal Receipt 74 x 500 mm
!define DEFAULT_PAPER_FORM "DMPAPER_LETTER"

# Available page orientations by default
# "PORTRAIT",          Portrait page orientation
# "LANDSCAPE",         Landscape page orientation
!define DEFAULT_PAGE_ORIENTATION "PORTRAIT"

Name "${PRINTER_NAME} virtual printer (x64)"
VIProductVersion "11.6.23.0"
VIAddVersionKey "ProductName" "${PRINTER_NAME}"
VIAddVersionKey "FileVersion" "11.6.23.0"
VIAddVersionKey "Comments" ""
VIAddVersionKey "CompanyName" "COMPANY_NAME"
VIAddVersionKey "LegalTrademarks" ""
VIAddVersionKey "LegalCopyright" ""
VIAddVersionKey "FileDescription" "Virtual printer setup"
OutFile "SetupPrn_x64.exe"
InstallDir ""

SilentInstall silent
SetCompressor lzma
AllowSkipFiles off
SetPluginUnload  alwaysoff

Icon printer.ico


; GetWindowsVersion 2.0 (2008-01-07)
;
; Based on Yazno's function, http://yazno.tripod.com/powerpimpit/
; Update by Joost Verburg
; Update (Macro, Define, Windows 7 detection) - John T. Haller of PortableApps.com - 2008-01-07
;
; Usage: ${GetWindowsVersion} $R0
;
; $R0 contains: 95, 98, ME, NT x.x, 2000, XP, 2003, Vista, 7 or '' (for unknown)

Function GetWindowsVersion

  Push $R0
  Push $R1

  ClearErrors

  ReadRegStr $R0 HKLM \
  "SOFTWARE\Microsoft\Windows NT\CurrentVersion" CurrentVersion

  IfErrors 0 lbl_winnt

  ; we are not NT
  ReadRegStr $R0 HKLM \
  "SOFTWARE\Microsoft\Windows\CurrentVersion" VersionNumber

  StrCpy $R1 $R0 1
  StrCmp $R1 '4' 0 lbl_error

  StrCpy $R1 $R0 3

  StrCmp $R1 '4.0' lbl_win32_95
  StrCmp $R1 '4.9' lbl_win32_ME lbl_win32_98

  lbl_win32_95:
    StrCpy $R0 '95'
  Goto lbl_done

  lbl_win32_98:
    StrCpy $R0 '98'
  Goto lbl_done

  lbl_win32_ME:
    StrCpy $R0 'ME'
  Goto lbl_done

  lbl_winnt:

  StrCpy $R1 $R0 1

  StrCmp $R1 '3' lbl_winnt_x
  StrCmp $R1 '4' lbl_winnt_x

  StrCpy $R1 $R0 3

  StrCmp $R1 '5.0' lbl_winnt_2000
  StrCmp $R1 '5.1' lbl_winnt_XP
  StrCmp $R1 '5.2' lbl_winnt_2003
  StrCmp $R1 '6.0' lbl_winnt_vista
  StrCmp $R1 '6.1' lbl_winnt_7 lbl_error

  lbl_winnt_x:
    StrCpy $R0 "NT $R0" 6
  Goto lbl_done

  lbl_winnt_2000:
    Strcpy $R0 '2000'
  Goto lbl_done

  lbl_winnt_XP:
    Strcpy $R0 'XP'
  Goto lbl_done

  lbl_winnt_2003:
    Strcpy $R0 '2003'
  Goto lbl_done

  lbl_winnt_vista:
    Strcpy $R0 'Vista'
  Goto lbl_done

  lbl_winnt_7:
    Strcpy $R0 '7'
  Goto lbl_done

  lbl_error:
    Strcpy $R0 ''
  lbl_done:

  Pop $R1
  Exch $R0

FunctionEnd

!macro GetWindowsVersion OUTPUT_VALUE
        Call GetWindowsVersion
        Pop `${OUTPUT_VALUE}`
!macroend

!define GetWindowsVersion '!insertmacro "GetWindowsVersion"'

# preinstallation stage
Function .onInit

        Var /GLOBAL inst_dir
        Var /GLOBAL action
        Var /GLOBAL result

        # get command line prarmeters
        ${GetParameters} $R0

        # parse /INSTDIR: parameter
        ${GetOptions} "$R0" "/INSTDIR:" $inst_dir
        IfErrors 0 +2
        StrCpy $result "abort"

        # parse /ACTION: parameter
        ${GetOptions} "$R0" "/ACTION:" $action
        IfErrors 0 +2
        StrCpy $result "abort"

        # check both parameters is specified
        StrCmp $result "abort" 0 +3
        MessageBox MB_OK "Invalid command line parameter."
        Quit

        # set the installation directory to the dir specified by /INSTDIR: parameter
        StrCpy $INSTDIR $inst_dir

FunctionEnd

Function ExitIfOSIsNotSupported
    ${GetWindowsVersion} $R0
    StrCmp $R0 "2000" +2
    Return
    MessageBox MB_OK "Sorry, this OS is not supported, driver cannot be installed."
    Abort
FunctionEnd

# main installation section
Section
        SetRegView 64

        StrCmp $action "INSTALL" install 0
        StrCmp $action "UNINSTALL" uninstall exit

  install:

        Call ExitIfOSIsNotSupported

    # extract driver files:

        SetOutPath $TEMP\~${PREFIX}

        ## common driver files
        File /oname=unidrv_${PREFIX}.dll   unidrv_7x64.dll
        File /oname=unidrv_${PREFIX}.hlp   unidrv_7x64.hlp
        File /oname=unidrvui_${PREFIX}.dll unidrvui_7x64.dll
        File /oname=unires_vpd.dll  unires_7x64.dll
        File /oname=stdnames_vpd.gpd stdnames_7x64.gpd
        File setupdrv.exe
        File ${PREFIX}.lng
        File ${PREFIX}pm.dll
        File ${PREFIX}.gpd
        File ${PREFIX}ui.dll
        File ${PREFIX}.ini
        File vpd_sdk.dll
        File vcredist_2013_x64.exe
        File vcredist_2015_x64.exe

        # install VC redistributable package
        ExecWait '"$TEMP\~${PREFIX}\vcredist_2013_x64.exe" /Q'
        ExecWait '"$TEMP\~${PREFIX}\vcredist_2015_x64.exe" /Q'

        # install printer driver
        ClearErrors
        ExecWait '"$TEMP\~${PREFIX}\setupdrv.exe" install defaultPaperForm "${DEFAULT_PAPER_FORM}" defaultPageOrientation "${DEFAULT_PAGE_ORIENTATION}"' $0
        StrCmp $0 0 +3 0
        SetErrorLevel $0
        Goto exit

        # extract remaining files to the specified installtion directory:

        SetOutPath $INSTDIR
        File VPDAgent_x64.exe
        File properties.exe
        File progressbar.exe
        File vpdisp.exe
        File fwproc_x64.exe
        File srvinst_x64.exe
        File ${PREFIX}.lng
        File emf2pdf.dll
        File pdfout.dll
        File /oname=MessageBox.exe MessageBox_x64.exe
        File vpd_sdk.dll


        # set up the agent service
        ClearErrors
        ExecWait '"$INSTDIR\srvinst_x64.exe" install' $0
        StrCmp $0 0 +4 0
        ExecWait '"$TEMP\~${PREFIX}\setupdrv.exe" uninstall'
        SetErrorLevel $0
        Goto exit

        #setup PDFCreator
        #ExecWait 'msiexec /i $TEMP\~${PREFIX}\redist.msi /qn'

        # remove temporary extracted driver files
        RMDir /r $TEMP\~${PREFIX}

        Goto exit

  uninstall:

    # unistall the printer and printer driver
    SetOutPath $TEMP\~${PREFIX}
        File setupdrv.exe
        #File redist.msi
        ExecWait '"$TEMP\~${PREFIX}\setupdrv.exe" uninstall'
        #ExecWait 'msiexec /x $TEMP\~${PREFIX}\redist.msi /qn'
        nsExec::Exec '"$INSTDIR\srvinst_x64.exe" uninstall'
        RMDir /r $TEMP\~${PREFIX}

        Delete $INSTDIR\VPDAgent_x64.exe
        Delete $INSTDIR\vpdisp.exe
        Delete $INSTDIR\properties.exe
        Delete $INSTDIR\progressbar.exe
        Delete $INSTDIR\${PREFIX}.lng
        Delete $INSTDIR\fwproc_x64.exe
        Delete $INSTDIR\srvinst_x64.exe
        Delete $INSTDIR\emf2pdf.dll
        Delete $INSTDIR\pdfout.dll
        Delete $INSTDIR\vpd_sdk.dll
        Delete $INSTDIR\MessageBox.exe

        # delete installation directory (only if it's empty)
        RMDir $INSTDIR

  exit:

SectionEnd
